/*
 * FILE:
 *   CoercriCallbacks.cs
 *
 * PURPOSE:
 *   Implementation of VM callbacks for Coercri
 *
 * AUTHOR:
 *   Stephen Thompson <stephen@solarflare.org.uk>
 *
 * CREATED:
 *   18-Jun-2011
 *
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2011.
 *
 *   This file is part of Mips2cs. Mips2cs is distributed under the terms
 *   of the Boost Software License, Version 1.0, the text of which
 *   appears below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *   
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 */

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using System.Diagnostics;
using System.Threading;

class CoercriCallbacks : MipsCallbacks
{
    CoercriGfx gfx;

    Stopwatch sw;

    public CoercriCallbacks(CoercriGfx g)
    {
        gfx = g;

	sw = new Stopwatch();
	sw.Start();
    }
    
    // int vm_write(const char *addr, int len)
    public int f_10(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        for (int i = 0; i < a1; ++i)
        {
            int Byte = mips.ReadByteU(a0);
            a0++;
            if (Byte == 10)
            {
                System.Console.WriteLine();
            }
            else
            {
                System.Console.Write((char)Byte);
            }
        }
        return a1;
    }

    // int vm_read(char *addr, int len)
    public int f_14(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        if (a1 == 0) return 0;  // zero sized read
        mips.WriteByte(a0, System.Console.Read());
        return 1;
    }

    // int vm_get_num_full_screen_modes()
    public int f_20(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        return gfx.GetNumFullScreenModes();
    }

    // void vm_get_full_screen_modes(void *addr)
    public int f_24(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.GetFullScreenModes(mips, a0);
        return 0;
    }

    // void vm_get_desktop_mode(void *addr)
    public int f_28(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.GetDesktopMode(mips, a0);
        return 0;
    }

    // void vm_create_window(void *win)
    //  --> win points to:
    //     width (4 bytes)
    //     height (4 bytes)
    //     resizable flag (4 bytes)
    //     fullscreen flag (4 bytes)
    //     title string addr (4 bytes)
    public int f_2c(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.CreateWindow(mips,
                         mips.ReadWord(a0),
                         mips.ReadWord(a0 + 4),
                         mips.ReadWord(a0 + 8) != 0,
                         mips.ReadWord(a0 + 12) != 0,
                         mips.ReadWord(a0 + 16));
        return 0;
    }

    // void vm_switch_window(int new_width, int new_height, int fullscreen_flag)
    public int f_30(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.SwitchWindow(a0, a1, a2 != 0);
        return 0;
    }

    // int vm_is_full_screen()
    public int f_34(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        return gfx.IsFullScreen() ? 1 : 0;
    }

    // int vm_create_graphic(void *addr)
    //  --> addr points to:
    //     pixel data addr (4 bytes)
    //     width (4 bytes)
    //     height (4 bytes)
    //     handle x (4 bytes)
    //     handle y (4 bytes)
    //  --> return value: ID number of the graphic
    public int f_38(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        return gfx.CreateGraphic(mips,
                                 mips.ReadWord(a0),
                                 mips.ReadWord(a0 + 4),
                                 mips.ReadWord(a0 + 8),
                                 mips.ReadWord(a0 + 12),
                                 mips.ReadWord(a0 + 16));
    }

    // void vm_destroy_graphic(int id)
    public int f_3c(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.DestroyGraphic(a0);
        return 0;
    }

    // void vm_get_window_size(int *width, int *height)
    public int f_40(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.GetWindowSize(mips, a0, a1);
        return 0;
    }

    // void vm_set_clip_rectangle(int x, int y, int w, int h)
    public int f_44(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.SetClipRectangle(a0, a1, a2, a3);
        return 0;
    }

    // void vm_clear_clip_rectangle()
    public int f_48(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.ClearClipRectangle();
        return 0;
    }

    // void vm_get_clip_rectangle(MipsVM mips, int *x, int *y, int *w, int *h)
    public int f_4c(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        int x=0, y=0, w=0, h=0;
        gfx.GetClipRectangle(ref x, ref y, ref w, ref h);
        mips.WriteWord(a0, x);
        mips.WriteWord(a1, y);
        mips.WriteWord(a2, w);
        mips.WriteWord(a3, h);
        return 0;
    }

    // void vm_clear_screen(int col)
    public int f_50(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.ClearScreen(DecodeColor(a0));
        return 0;
    }

    // void vm_plot_pixel(int x, int y, int col)
    public int f_54(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.PlotPixel(a0, a1, DecodeColor(a2));
        return 0;
    }

    // void vm_draw_line(void *ptr)
    //  --> ptr points to:
    //     x1, y1, x2, y2, col (all 4 bytes)
    public int f_58(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.DrawLine(mips.ReadWord(a0),
                     mips.ReadWord(a0 + 4),
                     mips.ReadWord(a0 + 8),
                     mips.ReadWord(a0 + 12),
                     DecodeColor(mips.ReadWord(a0 + 16)));
        return 0;
    }

    // void vm_fill_rectangle(void *ptr)
    //  --> same input as vm_draw_line
    public int f_5c(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.FillRectangle(mips.ReadWord(a0),
                          mips.ReadWord(a0 + 4),
                          mips.ReadWord(a0 + 8),
                          mips.ReadWord(a0 + 12),
                          DecodeColor(mips.ReadWord(a0 + 16)));
        return 0;
    }

    // void vm_draw_graphic(int x, int y, int id)
    public int f_60(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.DrawGraphic(a0, a1, a2);
        return 0;
    }

    // int vm_get_graphic_width(int id)
    public int f_64(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        return gfx.GetGraphicWidth(a0);
    }

    // int vm_get_graphic_height(int id)
    public int f_68(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        return gfx.GetGraphicHeight(a0);
    }

    // void vm_get_graphic_handle(int id, int *hx, int *hy)
    public int f_6c(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        int hx=0, hy=0;
        gfx.GetGraphicHandle(a0, ref hx, ref hy);
        mips.WriteWord(a1, hx);
        mips.WriteWord(a2, hy);
        return 0;
    }

    // void vm_get_graphic_pixels(int id, void *data)
    public int f_70(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.GetGraphicPixels(mips, a0, a1);
        return 0;
    }

    // void vm_end_batches()
    public int f_74(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        gfx.EndBatches();
        return 0;
    }

    private Color DecodeColor(int x)
    {
        return new Color((x & 0xff000000) >> 24,
                         (x & 0xff0000) >> 16,
                         (x & 0xff00) >> 8,
                         x & 0xff);
    }

    // unsigned int vm_get_msec()
    public int f_78(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        return (int) sw.ElapsedMilliseconds;
    }

    // void vm_sleep(int msec)
    public int f_7c(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        Thread.Sleep(a0);
        return 0;
    }

    // void vm_get_mouse_pos(int *x, int *y)
    public int f_80(MipsVM mips, int a0, int a1, int a2, int a3)
    {
        MouseState m = Mouse.GetState();
        mips.WriteWord(a0, m.X);
        mips.WriteWord(a1, m.Y);
        return 0;
    }
}
