/*
 * FILE:
 *   vm_calls.h
 *
 * PURPOSE:
 *   C prototypes for the special "VM calls"
 *
 * AUTHOR:
 *   Stephen Thompson <stephen@solarflare.org.uk>
 *
 * CREATED:
 *   18-Jun-2011
 *
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2011.
 *
 *   This file is part of Mips2cs. Mips2cs is distributed under the terms
 *   of the Boost Software License, Version 1.0, the text of which
 *   appears below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *   
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *   
 */

#ifndef VM_CALLS_H
#define VM_CALLS_H

#ifdef __cplusplus
extern "C" {
#endif

    void vm_yield();   // 0x4
    
    int vm_get_num_full_screen_modes();    // 0x20
    void vm_get_full_screen_modes(void *addr);   // 0x24
    void vm_get_desktop_mode(void *addr);
    void vm_create_window(void *win);
    void vm_switch_window(int new_width, int new_height, int fullscreen_flag);
    int vm_is_full_screen();
    int vm_create_graphic(void *addr);
    void vm_destroy_graphic(int id);
    void vm_get_window_size(int *width, int *height);
    void vm_set_clip_rectangle(int x, int y, int w, int h);
    void vm_clear_clip_rectangle();
    void vm_get_clip_rectangle(int *x, int *y, int *w, int *h);
    void vm_clear_screen(int col);
    void vm_plot_pixel(int x, int y, int col);
    void vm_draw_line(void *ptr);
    void vm_fill_rectangle(void *ptr);
    void vm_draw_graphic(int x, int y, int id);
    int vm_get_graphic_width(int id);
    int vm_get_graphic_height(int id);
    void vm_get_graphic_handle(int id, int *hx, int *hy);
    void vm_get_graphic_pixels(int id, void *data);
    void vm_end_batches();
    unsigned int vm_get_msec();
    void vm_sleep(int msec);
    void vm_get_mouse_pos(int *x, int *y);
    
    
#ifdef __cplusplus
}
#endif

#endif
